# -*- coding: utf-8 -*-

__version__ = "1.1.4"

# "import flask" is wrapped in a try/catch clause for setup.py to get the
# version without requiring flask
try:
    import flask
    app = flask.Flask(__name__)
    from . import api
except ImportError:
    pass



# -*- coding: utf-8 -*-

from __future__ import with_statement
from flasfka import app, __version__
import flask
import kafka
import json


app.config.update(dict(
    HOSTS=["localhost:9092"],
    DEFAULT_GROUP="flasfka",
    CONSUMER_TIMEOUT=0.1,
    CONSUMER_LIMIT=100
))

app.config.from_envvar("FLASFKA_CONFIG", silent=True)


def get_kafka_client():
    if not hasattr(flask.g, "kafka_client"):
        flask.g.kafka_client = kafka.KafkaClient(app.config["HOSTS"])
    return flask.g.kafka_client


def get_kafka_producer():
    if not hasattr(flask.g, "kafka_producer"):
        client = get_kafka_client()
        flask.g.kafka_producer = kafka.KeyedProducer(client)
    return flask.g.kafka_producer


def get_kafka_consumer(group, topic):
    client = get_kafka_client()
    return kafka.SimpleConsumer(
        client, group, topic,
        iter_timeout=app.config["CONSUMER_TIMEOUT"]
    )


def produce(topic, messages, key=None):
    producer = get_kafka_producer()
    for message in messages:
        if type(message) not in (unicode, str):
            raise TypeError
        producer.send(topic, key, message.encode("utf-8"))
    producer.stop()


def consume(topic, group, limit):
    if group is None:
        group = app.config["DEFAULT_GROUP"].encode("utf-8")
    consumer = get_kafka_consumer(group, topic)
    res = {"group": group.decode("utf-8"), "messages": []}
    for n, message in enumerate(consumer):
        res["messages"].append(message.message.value.decode("utf-8"))
        if n >= limit - 1:
            break
    consumer.commit()
    consumer.stop()
    return res


@app.route("/<topic>/", methods=["GET", "POST"])
@app.route("/<topic>/<group_or_key>/", methods=["GET", "POST"])
def flasfka(topic, group_or_key=None):
    topic = topic.encode("utf-8")
    if group_or_key is not None:
        group_or_key = group_or_key.encode("utf-8")

    client = get_kafka_client()
    client.ensure_topic_exists(topic)

    if flask.request.method == "GET":
        limit = int(flask.request.args.get(
            "limit", app.config["CONSUMER_LIMIT"]
            ))
        group = group_or_key
        return flask.jsonify(consume(topic, group, limit))
    if flask.request.method == "POST":
        key = group_or_key
        data = flask.request.get_json(force=True)
        try:
            produce(topic, data["messages"], key)
            return flask.make_response(("", 204, {}))
        except (KeyError, TypeError):
            return flask.make_response((
                'expected format: {"messages": ["message1", ...]}',
                400,
                {}
            ))

# Snippet to attach the version to every request
@app.after_request
def add_version_header(response):
    response.headers["X-Flasfka-Version"] = __version__
    return response



# -*- coding: utf-8 -*-
"""
Push/Pull on Kafka over HTTP
"""

from setuptools import setup
from subprocess import check_output as run
from subprocess import CalledProcessError
import sys
import os


from flasfka import __version__

# Travis uploads our releases on pypi when the tests are passing and there
# is a new tag.
#
# When this happens, we want to make sure that the version in the code
# (see flasfka/__init___.py) is in sync with the git tag. This snippet
# performs the check.
if os.getenv("TRAVIS") is not None:
    try:
        GIT_VERSION = run(["git", "describe"]).decode().strip()
        if not GIT_VERSION.startswith(__version__):
            sys.exit("The git tag does not match the version. Please fix.")
    except CalledProcessError:
        pass

setup(
    name="flasfka",
    version=__version__,
    description=__doc__,
    long_description=open("README.rst").read(),
    author="Christophe-Marie Duquesne",
    author_email="chmd@chmd.fr",
    url="https://github.com/travel-intelligence/flasfka",
    download_url="https://github.com/travel-intelligence/flasfka/" +
                 "archive/%s.tar.gz" % __version__,
    packages=["flasfka"],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        ],
    install_requires=[
        "Flask >= 0.10",
        "kafka-python >= 0.9.2",
        ],
    scripts=["flasfka-serve"],
    zip_safe=False,
    include_package_data=True
)



# -*- coding: utf-8 -*-

from __future__ import with_statement
import unittest
import random
import string
import json
import os
from . import app


def random_ascii_string(n=20):
    return ''.join(random.SystemRandom().choice(string.ascii_letters +
                   string.digits) for _ in range(n))


class FlafskaTestCase(unittest.TestCase):

    def setUp(self):
        app.config.update(dict(
            TESTING=True,
            ))
        self.app = app.test_client()

    def test_send_fetch(self):
        topic = random_ascii_string()
        message = {"messages": [random_ascii_string()]}

        response = self.app.post("/" + topic + "/",
                                 data=json.dumps(message))
        self.assertEqual(response.status_code, 204)

        response = self.app.get("/" + topic + "/")
        self.assertEqual(response.status_code, 200)
        body = json.loads(response.data)
        self.assertEqual(body, {u'group': u'flasfka', u'messages':
                         message["messages"]})

    def test_send_fetch_group(self):
        topic = random_ascii_string()
        message = {"messages": [random_ascii_string()]}
        group = random_ascii_string()

        response = self.app.post("/" + topic + "/",
                                 data=json.dumps(message))
        self.assertEqual(response.status_code, 204)

        response = self.app.get("/" + topic + "/" + group + "/")
        self.assertEqual(response.status_code, 200)
        body = json.loads(response.data)
        self.assertEqual(body, {u'group': group, u'messages':
                         message["messages"]})

    def test_send_non_json_data(self):
        message = '\xe5\xa2\xbe\xaa\xa7\xf8\xdf\x00\xc6\xbd'
        topic = random_ascii_string()

        response = self.app.post("/" + topic + "/", data=message)
        self.assertEqual(response.status_code, 400)

    def test_send_fetch_2_groups(self):
        topic = random_ascii_string()
        message = {"messages": [random_ascii_string()]}

        response = self.app.post("/" + topic + "/",
                                 data=json.dumps(message))
        self.assertEqual(response.status_code, 204)

        group = "group_1"
        response = self.app.get("/" + topic + "/" + group + "/")
        self.assertEqual(response.status_code, 200)
        body = json.loads(response.data)
        self.assertEqual(body, {u'group': group, u'messages':
                         message["messages"]})

        group = "group_2"
        response = self.app.get("/" + topic + "/" + group + "/")
        self.assertEqual(response.status_code, 200)
        body = json.loads(response.data)
        self.assertEqual(body, {u'group': group, u'messages':
                         message["messages"]})

    def test_send_fetch_same_group(self):
        topic = random_ascii_string()
        message = {"messages": [random_ascii_string()]}

        response = self.app.post("/" + topic + "/",
                                 data=json.dumps(message))
        self.assertEqual(response.status_code, 204)

        group = "group"
        response = self.app.get("/" + topic + "/" + group + "/")
        self.assertEqual(response.status_code, 200)
        body = json.loads(response.data)
        self.assertEqual(body, {u'group': group, u'messages':
                         message["messages"]})

        response = self.app.get("/" + topic + "/" + group + "/")
        self.assertEqual(response.status_code, 200)
        body = json.loads(response.data)
        self.assertEqual(body, {u'group': group, u'messages': []})

    def test_limit(self):
        topic = random_ascii_string()
        message = {"messages": ["hello %i" % i for i in range(400)]}

        response = self.app.post("/" + topic + "/",
                                 data=json.dumps(message))
        self.assertEqual(response.status_code, 204)

        for _ in range(20):
            response = self.app.get("/" + topic + "/" + "?limit=10")
            self.assertEqual(response.status_code, 200)
            body = json.loads(response.data)
            self.assertLessEqual(len(body['messages']), 10)

    def test_send_wrong_messages(self):
        topic = random_ascii_string()
        message = {"messages": [{"a": "dictionary"}]}

        response = self.app.post("/" + topic + "/",
                                 data=json.dumps(message))
        self.assertEqual(response.status_code, 400)
